#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyPrintwrap ':all';


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $x_max = 0;
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  print "  \\coordinate (start) at ($x,$y);\n";
  $x_max = max($x_max, $x);
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
  $x_max = max($x_max, $x);
}

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
my $D = 0;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;  # within $n_hi
  next if @n_list == 1;      # skip singles
  next if $n != $n_list[0];  # only the visit
  printwrap("($x,$y),");
  $D++;
  $x_max = max($x_max, $x);
}
printwrap("}");
printwrap("{ \\p node[my dot] };\n");

print "  \\coordinate (x-max) at ($x_max,0);\n";
print "  \\newcommand{\\MyD}{$D}\n";
print "% GP-Test-Last  D($level) == $D\n";

print "  % end generated\n";
exit 0;
