#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'dir','Hwpf_box_points';
use MyLineSegments;
use MyPlanar;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 2;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $box_points = Hwpf_box_points();

my $hull_points = $box_points;
my $ex = 1;
my $ey = 0;
foreach (1 .. $level) {
  ($hull_points, $ex,$ey) = MyR5Dragon::hull_expand($hull_points, $ex,$ey);
}

sub translate_box {
  my ($x,$y, $dir) = @_;
  my $points = MyPlanar::points_rotate_plus90($box_points, $dir);
  return MyPlanar::points_move($points, $x,$y);
}

{
  $Printwrap = 2;
  printwrap("\\draw[my boxes] ");
  $Printwrap += 2;

  my $segments = MyLineSegments->new;
 N: foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    $segments->add($x,$y, $path->n_to_xy($n+1));

    my $points = translate_box($x,$y, dir($n));
    if (MyPlanar::points_has_boundary_touch($points,$hull_points)) {
      foreach my $i (0 .. $#$points) {
        if (MyPlanar::poly_has_boundary_point($hull_points, $points->[$i])
            && MyPlanar::poly_has_boundary_point($hull_points, $points->[$i-1])) {
          # side $i-1 to $i on hull, don't draw it
          my $join = '';
          foreach my $p (@{$points}[$i..$#$points],
                         @{$points}[0 .. $i-1]) {
            my ($x,$y) = @$p;
            $x = sprintf '%.8g', $x;
            $y = sprintf '%.8g', $y;
            printwrap("$join($x,$y)");
            $join = '--';
          }
          next N;
        }
      }

      # $points rectangle no sides common to $hull_points, draw all
      my $join = '';
      foreach my $p (@$points) {
        my ($x,$y) = @$p;
        $x = sprintf '%.8g', $x;
        $y = sprintf '%.8g', $y;
        printwrap("$join($x,$y)");
        $join = '--';
      }
      printwrap("${join}cycle");
    }
  }
  printwrap(";\n");

  ### $segments
  ### lines: $segments->lines_joined

  $Printwrap = 2;
  printwrap("\\draw[my lines] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap_newline();
}

# print scalar(@$points)," points, end $ex,$ey\n";
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
foreach my $p (@$hull_points) {
  my ($x,$y) = @$p;
  $x = sprintf '%.8g', $x;
  $y = sprintf '%.8g', $y;
  printwrap("$join($x,$y)");
  $join = '--';
}
printwrap("${join}cycle;");
printwrap(" % ",scalar(@$hull_points)," points\n");

print "  % end generated\n";
exit 0;
