#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph;

use FindBin;
use lib $FindBin::Bin;
use MyGraphs;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
use Smart::Comments;

my $min_s = 3;
my $max_s = 6;
my $max_k = 2;
my $max_vertices = 20;

if (@ARGV && $ARGV[0] eq 'big') {
  $min_s = 3;
  $max_s = 20;
  $max_k = 6;
  $max_vertices = 80;
}

print "% generated by tools/$FindBin::Script\n";

my %tables;
foreach my $s ($min_s .. $max_s) {
  # print "N=$s\n";
  my $graph = Graph->new (undirected => 1);
  $graph->add_vertex(0);
  my $join = '';
  foreach my $k (0 .. $max_k) {
    my @path = MyGraphs::Graph_tree_diameter_path($graph);
    my $diameter = scalar(@path) - 1;
    $diameter == 3**$k-1 or die;

    my $num_vertices = $graph->vertices;
    $num_vertices == $s**$k or die;

    $tables{'W'}->[$s]->[$k] = MyGraphs::Graph_Wiener_index($graph);
    $tables{'WC'}->[$s]->[$k] = MyGraphs::Graph_Wiener_part_at_vertex($graph,$path[0]);
    if ($graph->vertices > 60) {
      last;
    }
    $graph = MyGraphs::Graph_star_replacement($graph,$s);
    $join = ',';
  }
}

printwrap_indent("% GP-DEFINE  ");
# ### %tables
foreach my $key (sort keys %tables) {
  my $table = $tables{$key};
  my $prefix = "gen_StarRep${key}_samples = {[";
  $Printwrap = 0;
  foreach my $i ($min_s .. $#$table) {
    my $row = $table->[$i] || [];
    printwrap("${prefix}["
              . join(',',@$row)
              . "]"
              . ($i==$#$table ? '' : ',')
              . "\n");
    $prefix = '  ';
    $Printwrap = 2;
  }
  printwrap "]};\n";
}

print "% end generated\n";
exit 0;
