#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'chamfer','xy_rotate_minus90','xy_rotate_plus90';
use MyPrintwrap ':all';

use Math::PlanePath;
*_divrem_mutate = \&Math::PlanePath::_divrem_mutate;


print "  % generated by tools/$FindBin::Script\n";

my $n_hi = 53;

sub path_n_to_enc {
  my ($path, $n, $side) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($dxS,$dyS) = ($side eq 'left'
                    ? xy_rotate_plus90($dx,$dy)
                    : xy_rotate_minus90($dx,$dy));
  return
    (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxS,$y+$dyS, $n)
     && path_xyxy_is_traversed_either($path, $x2,$y2, $x2+$dxS,$y2+$dyS, $n)
     && path_xyxy_is_traversed_either($path, $x+$dxS,$y+$dyS, $x2+$dxS,$y2+$dyS, $n));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

sub enc {
  my ($delta) = @_;
  if ($delta == 0) { return ''; }
  if ($delta == 1) { return '+\MyEnc'; }
  if ($delta == -1) { return '-\MyEnc'; }
  die;
}

my $path = Math::PlanePath::R5DragonCurve->new;
foreach my $side ('right', 'left') {

  print "  \\begin{scope}[shift=(pos-$side)]\n";
  print "    \\node (start-$side) at (0,0) [my dot];\n";
  $Printwrap=4;
  printwrap("\\draw [my thin] (start-$side)");
  $Printwrap=6;

  foreach my $n (1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    {
      # before $n
      my ($dx,$dy) = $path->n_to_dxdy($n-1);
      printwrap("--($x".chamfer(-$dx).",$y".chamfer(-$dy).")");
    }
    {
      # after $n
      my ($dx,$dy) = $path->n_to_dxdy($n);
      printwrap("--($x".chamfer($dx).",$y".chamfer($dy).")");
    }
  }
  {
    # part-way into segment after $n_hi
    my $n = $n_hi;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    printwrap("--($x+\\MyEnd*$dx,$y+\\MyEnd*$dy)");
  }
  printwrap(";\n");

  my @values;
  $Printwrap=4;
  printwrap("\\draw [my thick]");
  $Printwrap=6;
  foreach my $n (0 .. $n_hi-1) {
    my $enc = path_n_to_enc($path,$n,$side);
    push @values, $enc ? 1 : 0;
    next unless $enc;

    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    printwrap(" ($x".chamfer($dx,'\\MyEnc')
              . ",$y".chamfer($dy,'\\MyEnc').")");

    $x += $dx;
    $y += $dy;
    printwrap("--($x".enc(-$dx).",$y".enc(-$dy).")");
  }
  printwrap(";\n");

  my $num_values = scalar @values;
  my $RL = uc(substr($side,0,1));
  print "% GP-Test-Last  vector($num_values,n,n--; Epred$RL(n)) == [",join(',',@values),"];\n";
  print "  \\end{scope}\n";
}
print "  % end generated\n";
exit 0;
