#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::BaseCnv;
use Math::BigInt;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyFLAT;
$|=1;


print "% generated by tools/$FindBin::Script\n";
my $max_len = Math::BigInt->new(10);

{
  MyR5Dragon::fBpred_hullstouch_init();
  MyFLAT::FLAT_print_gp_inline_table($MyR5Dragon::fNonBpred_hullstouch_flat,
                                     "fBpred_hullstouch_state_table");
  foreach my $f ($MyR5Dragon::fNonRpred_hullstouch_flat,
                 $MyR5Dragon::fNonLpred_hullstouch_flat,
                 $MyR5Dragon::fNonBpred_hullstouch_flat) {
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    MyFLAT::FLAT_print_gp_inline_accepting($f, "${name}_accepting");
  }

  foreach my $f ($MyR5Dragon::fNonRpred_hullstouch_flat,
                 $MyR5Dragon::fNonLpred_hullstouch_flat,
                 $MyR5Dragon::fNonBpred_hullstouch_flat) {
    join(',',$f->get_starting) == '0' or die "oops";
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    print "% GP-DEFINE  ${name}_counts = [",
      join(',',MyFLAT::FLAT_count_contains($f,$max_len)),"];\n";
  }

  foreach my $f ($MyR5Dragon::fNonRpred_hullstouch_flat,
                 $MyR5Dragon::fNonLpred_hullstouch_flat,
                 $MyR5Dragon::fNonBpred_hullstouch_flat) {
    join(',',$f->get_starting) == '0' or die "oops";
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    $f = $f->MyFLAT::minimize;
    print "% GP-DEFINE  ${name}_minimized_size = ",$f->num_states,";\n";
  }
}

print "% end generated\n";
exit 0;
