#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Tie::IxHash;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

my $n_start = 4*$n_hi;
my $n_end   = 8*$n_hi;
my ($ox,$oy) = $path->n_to_xy($n_start);

my $segments = MyLineSegments->new;
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add ($x-$ox, $y-$oy, $x2-$ox, $y2-$oy);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

{
  my ($x,$y) = $path->n_to_xy($n_start);
  $x -= $ox;
  $y -= $oy;
  print "  \\node (start) at ($x,$y) [my dot];\n";
}
foreach my $m (5 .. 7) {
  my ($x,$y) = $path->n_to_xy($m*$n_hi);
  $x -= $ox;
  $y -= $oy;
  print "  \\node at ($x,$y) [my dot];\n";
}

print "  % end generated\n";
exit 0;
