\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Work in progress ... going to change wildly.
\\
\\     p = geom_conj(p)
\\
\\     v = geom_bbox_of_points(v)
\\     M = geom_inertia_of_intformal_line(tmin,tmax, xf,yf,zf, density=1)
\\     M = geom_inertia_scale_line(M,f)
\\     M = geom_inertia_scale_area(M,f)
\\
\\     s = geom_turn_LSR(p1,p2,p3)
\\
\\     bool = geom_is_point_colinear(p1,p2)
\\     p = geom_projection(p,q)                   OR default q2=0   
\\     bool = geom_is_point_on_line(p, q1,q2)
\\     p = geom_projection_on_line(p, q1,q2)
\\     d = geom_distance_to_line(p, q1,q2)
\\
\\     z = geom_complex_X_intercept(z1,z2)      projection onto 1
\\     z = geom_complex_Y_intercept(z1,z2)      projection onto I
\\
\\ End Summary

\\ Maybe:
\\ geom_line_is_colinear(p1,p2, q1,q2)
\\ geom_distance_to_segment(p, q1,q2)
\\ geom_is_on_line(p, q1,q2)             \\ distance==0
\\ v = geom_hull_of_points(v)
\\ bool = geom_is_inside_polygon(p,v)


\\ FIXME: might like to keep point type always
geom_bbox_of_points(v) =
{
  if(#v<3, v,
     my(p=apply(geom_point_x,v), Xmin=vecmin(p), Xmax=vecmax(p));
     p=apply(geom_point_y,v); my(Ymin=vecmin(p), Ymax=vecmax(p));
     [[Xmin,Ymin], [Xmax,Ymin], [Xmax,Ymax], [Xmin,Ymax]]);
}
{
addhelp(geom_bbox_of_points,
"v = geom_bbox_of_points(v)
v is a vector of points in the X,Y plane.
Return a polygon which is the bounding box around them.
The polygon points are Xmin,Ymin, Xmax,Ymin, Xmax,Ymax, Xmin,Ymax, so 
anti-clockwise around from Xmin,Ymin.
If v is an empty vector then return an empty polygon.");
}

geom_projection(p,q) = geom_dot_product(p,q) * q / geom_norm(q);
{
addhelp(geom_projection,
"p = geom_projection(p,q)
Return the point of perpendicular projection of point p onto the line
through point q and the origin.");
}

geom_projection_on_line(p, q1,q2) = geom_projection(p-q1,q2-q1) + q1;
{
addhelp(geom_projection_on_line,
"p = geom_projection_on_line(p, q1,q2)
Return the point of perpendicular projection of point p onto the line
through points q1,q2.");
}

geom_distance_to_line(p, q1,q2) = \
  geom_length_of_segment(p, geom_projection_on_line(p,q1,q2));
{
addhelp(geom_distance_to_line,
"d = geom_distance_to_line(p, q1,q2)
Return the distance from point p to the line through points q1,q2.
Distance is by perpendicular projection (geom_projection_on_line()), so is
the shortest distance to the line.");
}

geom_turn_LSR(p1,p2,p3) =
{
  my(p = p3-p1,
     xydir = p2-p1);
  sign(  geom_point_y(p)*geom_point_x(xydir)
       - geom_point_x(p)*geom_point_y(xydir));
}
{
addhelp(geom_turn_LSR,
"s = geom_turn_LSR(p1,p2,p3)
p1,p2,p3 are points in the XY plane.  p1,p2 are a direction and p3 is the
next point.  Return
   1 if p3 is a turn to the left of p1,p2
   0 if straight ahead or reverse
  -1 if a turn to the right.");
}

geom_conj(p) =
{
  my(t = type(p));
  if(t=="t_VEC" || t=="t_COL", p[2]   = -p[2];   p,
     t=="t_MAT",               p[2,1] = -p[2,1]; p,
     conj(p));
}
{
addhelp(geom_conj,
"p = geom_conj(p)
Return the conjugate of xy direction p.  This is the usual conj(p) for a
complex or quad, or negation of Y coordinate in a point vector.");
}


\\-----------------------------------------------------------------------------
\\ not sure these are worth having

geom_inertia_scale_line(M,f) = M*f^3;
{
addhelp(geom_inertia_scale_line,
"M = geom_inertia_scale_line(M,f)
M is a 3x3 inertia matrix of some line segments.
Return a new inertia matrix for the shape with all X,Y,Z coordinates scaled
by factor f and mass proportional to the lengthened line segments.
This is a factor f^3 on each inertia component, that being f^2 for
coordinates squared, and f for mass in the new length.");
}

geom_inertia_scale_area(M,f) = M*f^4;
{
addhelp(geom_inertia_scale_area,
"M = geom_inertia_scale_area(M,f)
M is a 3x3 inertia matrix of some area.
Return a new inertia matrix for the shape with all X,Y,Z coordinates scaled
by factor f and mass proportional to the new area (density unchanged).
This is a factor f^4 on each inertia component, that being f^2 for
coordinates squared, and f^2 for mass in the new area.");
}


\\-----------------------------------------------------------------------------
\\ Line Intersections

geom_INTERNAL_coordinate_funcs = [geom_point_x,geom_point_y,geom_point_z];

geom_is_point_colinear(p1,p2) =
{
  my(s1=0, s2=0);
  for(i=1,3,
     my(x1=geom_INTERNAL_coordinate_funcs[i](p1),
        x2=geom_INTERNAL_coordinate_funcs[i](p2));
     \\ print("i="i" "x1" "x2"  scale "s1" "s2);
     x1*s1 == x2*s2 || return(0);
     if(x1 || x2,
        s1=x2; s2=x1));
  1;
}
{
addhelp(geom_is_point_colinear,
"bool = geom_is_point_colinear(p1,p2)
Return 1 if points p1 and p2 are colinear in the sense of being the same
direction but possibly different magnitudes.");
}

geom_is_point_on_line(p,q1,q2) = geom_is_point_colinear(p-q1,q2-q1);
{
addhelp(geom_is_point_on_line,
"bool = geom_is_point_on_line(p, q1,q2)
Return 1 if point p is on the line passing through points q1,q2, or return 0
if not.

The test is for p exactly on the line so usually all coordinates should be
exact types.  See geom_distance_to_line() to test for something close enough
when floating point rounding.");
}

\\ |         z1
\\ |        /
\\ |      z2
\\ |      /
\\ |     /
\\ *----E--------
\\
\\  imag(z1) / imag(z2) = (real(z1)-E) / (real(z2)-E)
\\  E + (real(z2)-E) * imag(z1) / imag(z2) = real(z1)
\\  E - E*imag(z1)/imag(z2) = real(z1) - real(z2)*imag(z1)/imag(z2)
\\  (z1-E)/(z2-E) == imag(z1) / imag(z2)
\\  real(z1) - imag(z1) / imag(z2) * real(z1)
\\
geom_complex_X_intercept(z1,z2) =
{
  \\ z1 - (z1-z2) * imag(z1) / imag(z1-z2);
  \\ (real(z1)*imag(z1-z2) - real(z1-z2)*imag(z1)) / imag(z1-z2);
  \\ z1 - (z1-z2) * imag(z1) / (imag(z1) - imag(z2));
  \\ real(z1) - (real(z1) - real(z2)) * imag(z1) / (imag(z1) - imag(z2));
  (geom_point_x(z2)*geom_point_y(z1) - geom_point_x(z1)*geom_point_y(z2))
  / (geom_point_y(z1) - geom_point_y(z2));
}

geom_complex_Y_intercept(z1,z2) =
{
  (geom_point_y(z2)*geom_point_x(z1) - geom_point_y(z1)*geom_point_x(z2))
  / (geom_point_x(z1) - geom_point_x(z2));
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_intformal_line()

\\ density might better a function of length along, unless would write x,y,z
\\ in terms of a length t

\\ maybe point p = [xf,yf,zf]
\\ maybe default zf=0
geom_inertia_of_intformal_line(tmin,tmax, xf,yf,zf, density=1) =
{
  my(t_intformal =
     (poly)->
     poly = intformal(poly*density, 't);
     subst(poly,'t,tmax) - subst(poly,'t,tmin));

  my(Ix = t_intformal(yf*yf + zf*zf),
     Iy = t_intformal(xf*xf + zf*zf),
     Iz = t_intformal(xf*xf + yf*yf),
     Ixy = t_intformal(xf*yf),
     Ixz = t_intformal(xf*zf),
     Iyz = t_intformal(yf*zf));
  [ Ix, -Ixy, -Ixz;
   -Ixy, Iy,  -Iyz;
   -Ixz, -Iyz,  Iz ];
}
{
addhelp(geom_inertia_of_intformal_line,
"M = geom_inertia_of_intformal_line(tmin,tmax, xf,yf,zf, density=1)
Return the inertia matrix for a line defined by an integral over a
formal parameter t.

xf,yf,zf are polynomials in variable 't which give the location of the line
at that t.  A line in just the XY plane can be made with zf=0.  Inertia is
calculated by suitably integrating xf,yf,zf over t.  For example Ix

        t=tmax  
  Ix = integral  density * (yf^2 + zf^2) dt
        t=tmin             

\"density\" is a polynomial in variable 't giving mass density as a function
of t.  The default density=1 is mass proportional to ...  and uniformly
distributed across the ...

The integrals are made with intformal() so tmin,tmax and density can be
polynomials in further variables too, for example some 'l as overall length.
This gives a parameterized final formula and is a typical usage.  A final
formula might then be massaged manually to make an inertia calculating
function (or just subst() into it).");
}


\\-----------------------------------------------------------------------------

\\ Local variables:
\\ compile-command: "gp -f --quiet <test-wip.gp"
\\ End:
